import numpy
from typing import ClassVar, overload

__version__: str

class AttachMode:
    __members__: ClassVar[dict] = ...  # read-only
    __entries: ClassVar[dict] = ...
    createNew: ClassVar[AttachMode] = ...
    dontTouch: ClassVar[AttachMode] = ...
    firstEatSecond: ClassVar[AttachMode] = ...
    morePointsEatLow: ClassVar[AttachMode] = ...
    secondEatFirst: ClassVar[AttachMode] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    @property
    def name(self): ...
    @property
    def value(self) -> int: ...

class Bar3dvalue:
    count: int
    value: Barscalar
    def __init__(self, *args, **kwargs) -> None: ...

class BarConstructor:
    def __init__(self) -> None: ...
    def addStructure(self, ProcType: ProcType, ColorType: ColorType, ComponentType: ComponentType) -> None: ...

class BarType:
    __members__: ClassVar[dict] = ...  # read-only
    BYTE8_1: ClassVar[BarType] = ...
    BYTE8_3: ClassVar[BarType] = ...
    FLOAT32_1: ClassVar[BarType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    @property
    def name(self): ...
    @property
    def value(self) -> int: ...

class Barcontainer:
    def __init__(self, *args, **kwargs) -> None: ...
    def addItem(self, Baritem: Baritem) -> None: ...
    def clone(self, *args, **kwargs): ...
    def count(self) -> int: ...
    def getItem(self, index: int) -> Baritem: ...
    def maxLen(self) -> Barscalar: ...
    def preprocessBarcode(self, threshold: Barscalar, normalize: bool) -> None: ...
    def relength(self) -> None: ...
    def removeByThreshold(self, threshold: Barscalar) -> None: ...
    def sum(self) -> Barscalar: ...
    def __len__(self) -> int: ...

class Baritem:
    def __init__(self, *args, **kwargs) -> None: ...
    def SortByLineLen(self) -> None: ...
    def SortByPointsCount(self) -> None: ...
    def calcHistByBarlen(self) -> list: ...
    def clone(self) -> Baritem: ...
    def cmp(self, bitem: Baritem, CompareStrategy: CompareStrategy) -> float: ...
    def compareBestRes(self, bitem: Baritem, CompareStrategy: CompareStrategy) -> float: ...
    def compareOccurrence(self, bitem: Baritem, CompareStrategy: CompareStrategy) -> float: ...
    def getBarcodeLines(self) -> list[Barline]: ...
    def getBettyNumbers(self) -> list: ...
    def getRootNode(self) -> Barline: ...
    def maxLen(self) -> Barscalar: ...
    def normalize(self) -> None: ...
    def preprocessBarcode(self, arg0: Barscalar, arg1: bool) -> None: ...
    def relength(self) -> None: ...
    def removeByThreshold(self, arg0: Barscalar) -> None: ...
    def sum(self) -> Barscalar: ...
    def __len__(self) -> int: ...

class Barline:
    def __init__(self, *args, **kwargs) -> None: ...
    def compare3dbars(self, arg0: Barline, arg1: CompareStrategy) -> float: ...
    def end(self) -> Barscalar: ...
    def get3dList(self) -> list: ...
    def get3dSize(self) -> int: ...
    def get3dValue(self, arg0: int) -> Bar3dvalue: ...
    def getBettyNumbers(self) -> list: ...
    def getChildren(self) -> list: ...
    def getMatrixSize(self) -> int: ...
    def getMatrixValue(self, arg0: int) -> MatrixValue: ...
    def getParent(self) -> Barline: ...
    def getPoints(self, skipChildPoints: bool = ...) -> list: ...
    def getRect(self) -> list: ...
    def len(self) -> Barscalar: ...
    def start(self) -> Barscalar: ...
    def __len__(self) -> int: ...

class Barscalar:
    @overload
    def __init__(self, arg0: int) -> None: ...
    @overload
    def __init__(self, arg0: float, arg1: BarType) -> None: ...
    @overload
    def __init__(self, arg0: int, arg1: BarType) -> None: ...
    @overload
    def __init__(self, arg0: int, arg1: int, arg2: int) -> None: ...
    @overload
    def __init__(self, arg0: int, arg1: int, arg2: int, arg3: int) -> None: ...
    def absDiff(self, arg0: Barscalar) -> Barscalar: ...
    def getAvgFloat(self) -> float: ...
    def getAvgUchar(self) -> int: ...
    def getByte8(self) -> int: ...
    def getFloat(self) -> float: ...
    def getRGB(self, arg0: int) -> int: ...
    def value(self) -> tuple: ...
    def __add__(self, arg0: Barscalar) -> Barscalar: ...
    def __eq__(self, arg0: Barscalar) -> bool: ...
    def __ge__(self, arg0: Barscalar) -> bool: ...
    def __gt__(self, arg0: Barscalar) -> bool: ...
    def __le__(self, arg0: Barscalar) -> bool: ...
    def __lt__(self, arg0: Barscalar) -> bool: ...
    def __mul__(self, arg0: Barscalar) -> Barscalar: ...
    def __ne__(self, arg0: Barscalar) -> bool: ...
    def __sub__(self, arg0: Barscalar) -> Barscalar: ...
    def __truediv__(self, arg0: Barscalar) -> Barscalar: ...

class ColorType:
    __members__: ClassVar[dict] = ...  # read-only
    __entries: ClassVar[dict] = ...
    gray: ClassVar[ColorType] = ...
    native: ClassVar[ColorType] = ...
    rgb: ClassVar[ColorType] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    @property
    def name(self): ...
    @property
    def value(self) -> int: ...

class CompareStrategy:
    __members__: ClassVar[dict] = ...  # read-only
    CommonToLen: ClassVar[CompareStrategy] = ...
    CommonToSum: ClassVar[CompareStrategy] = ...
    __entries: ClassVar[dict] = ...
    compare3d: ClassVar[CompareStrategy] = ...
    compare3dBrightless: ClassVar[CompareStrategy] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    @property
    def name(self): ...
    @property
    def value(self) -> int: ...

class ComponentType:
    __members__: ClassVar[dict] = ...  # read-only
    Component: ClassVar[ComponentType] = ...
    Hole: ClassVar[ComponentType] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    @property
    def name(self): ...
    @property
    def value(self) -> int: ...

class MatrixValue:
    value: Barscalar
    x: int
    y: int
    def __init__(self) -> None: ...
    def getIndex(self) -> int: ...

class Point:
    x: int
    y: int
    def __init__(self, arg0: int, arg1: int) -> None: ...

class ProcType:
    __members__: ClassVar[dict] = ...  # read-only
    Radius: ClassVar[ProcType] = ...
    __entries: ClassVar[dict] = ...
    f0t255: ClassVar[ProcType] = ...
    f255t0: ClassVar[ProcType] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    @property
    def name(self): ...
    @property
    def value(self) -> int: ...

class ReturnType:
    __members__: ClassVar[dict] = ...  # read-only
    __entries: ClassVar[dict] = ...
    barcode2d: ClassVar[ReturnType] = ...
    barcode3d: ClassVar[ReturnType] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    @property
    def name(self): ...
    @property
    def value(self) -> int: ...

class barstruct:
    attachMode: AttachMode
    colorRange: int
    coltype: ColorType
    comtype: ComponentType
    createBinaryMasks: bool
    createGraph: bool
    killOnMaxLen: bool
    maxRadius: float
    proctype: ProcType
    returnType: ReturnType
    trueSort: bool
    def __init__(self) -> None: ...
    def setMaxLen(self, val: float) -> None: ...

def create(arg0: numpy.ndarray, arg1: barstruct) -> Baritem: ...
def createMultiple(arg0: numpy.ndarray, arg1: BarConstructor) -> Barcontainer: ...



import struct
from typing import BinaryIO, Optional, Callable, List

class bin_state_reader:
	def __init__(self, stream: Optional[BinaryIO] = None):
		self.main_stream = open(stream, 'rb') if isinstance(stream, str) else stream
		self.stream = self.main_stream
		self.inside = stream is None
		self.parse_barscalar = None  # Callable[[BinaryIO], Barscalar]
		self.memoffs = []
		self.items_end_pos = 0

		if self.stream:
			self.inti_on_open()

	def read_raw(self, fmt: str):
		size = struct.calcsize(fmt)
		data = self.stream.read(size)
		return struct.unpack(fmt, data)[0]

	def inti_on_open(self):
		self.items_end_pos = self.read_raw('Q')  # Assuming size_t is 8 bytes
		cur_pos = self.stream.tell()

		self.stream.seek(self.items_end_pos)

		size = self.read_raw('I')  # Assuming uint is 4 bytes
		self.memoffs = [self.read_raw('Q') for _ in range(size)]

		self.stream.seek(cur_pos)

	def open(self, path: str) -> bool:
		if not self.inside:
			raise Exception("Stream already initialized externally")

		self.main_stream = open(path, 'rb')
		self.stream = self.main_stream

		if self.main_stream:
			self.inti_on_open()
			return True
		else:
			return False

	def move_index(self, index: int):
		assert index < len(self.memoffs)
		self.stream.seek(self.memoffs[index])

	def get_index_size(self):
		return len(self.memoffs)

	def ended(self) -> bool:
		return self.stream.tell() >= self.items_end_pos or self.stream.peek() == b''

	def close(self) -> None:
		if not self.inside and self.main_stream is not None:
			self.main_stream.close()

	def p_bool(self):
		return self.read_raw('B') > 0

	def p_short(self):
		return self.read_raw('h')

	def p_int(self):
		return self.read_raw('i')

	def p_float(self):
		return self.read_raw('f')

	def p_int64(self):
		return self.read_raw('Q')

	def p_barscalar(self):
		return self.parse_barscalar(self.stream)

	def p_string(self):
		length = self.read_raw('H')  # Assuming ushort is 2 bytes
		return str(self.stream.read(length))

	def p_array(self):
		return self.p_int()



from setuptools import setup, find_packages

setup(
    name="ImageTopoDec",
    version="1.0.4",
    description="Python wrapper for the Barcode library.",
    author="Art A",
    author_email="your.email@example.com",
    url="https://github.com/Noremos/Barcode",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3.13",
        "Operating System :: OS Independent",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS",
    ],
    packages=find_packages(),
    include_package_data=True,
    python_requires="==3.13",
    install_requires=[
        "numpy",
        "matplotlib",
    ],
    package_data={
        "ImageTopoDec": [
            "libbarpy.so",
            "libbarpy.pyi",
            "libbarpy.pyd"
        ],
    },
    has_ext_modules = lambda: True,
)




import matplotlib.pyplot as plt
import numpy as np

from . import barcode as bc

def plot_barcode_lines(lines : set[bc.Barline] | bc.Baritem | bc.Barline, name, show = False, save = False):
	if isinstance(lines, bc.Barcode):
		reverse = lines.revert
		lines = lines.item.getBarcodeLines()
		if reverse:
			lines.reverse()

	elif isinstance(lines, bc.Baritem):
		lines = lines.getBarcodeLines()
	elif isinstance(lines, bc.Barline):
		lines = [lines]
	elif not isinstance(lines, list) and not isinstance(lines, set):
		raise Exception("plot_barcode_lines: wrong type of input data")

	# Количество объектов
	np.random.seed(0)  # Для повторяемости результатов
	# Уникальные цвета для каждого объекта
	colors = []
	for i in range(30):
		r = np.round(np.random.rand(),1)
		g = np.round(np.random.rand(),1)
		b = np.round(np.random.rand(),1)

		colors.append([r,g,b])

	separateFig, ax = plt.subplots(1, 1)
	ax.grid(True)

	ax.set_xlabel('Brightness')
	ax.set_ylabel('Component number')

	k = 1
	for line in lines:
		x_coords = [line.start().getAvgUchar(), line.end().getAvgUchar()]
		heights = [k, k]
		ax.plot(x_coords, heights, linestyle='-', color=colors[k % len(colors)], linewidth= 1)
		k += 1

	if show:
		plt.show()

	if save:
		plt.savefig(f"{name}.png", dpi=300)
		plt.savefig(f"{name}.svg")


def plot_3d_list(item : bc.Baritem):
    bar = item.getBarcodeLines()

    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')

    n = 100

    kn=0
    for bl in bar:
        vals = bl.get3dList()

        for i in range(0,len(vals),5):
            ax.scatter(kn, vals[i].value, vals[i].count, marker='o')
        kn+=1

    ax.set_xlabel('X Label')
    ax.set_ylabel('Y Label')
    ax.set_zlabel('Z Label')

    plt.show()


from .libbarpy import *  # Import symbols from the .pyd file
import numpy as np


def append_line_to_matrix(barline : Barline, matrix : np.array):
	ksize = barline.getMatrixSize()
	for i in range(ksize):
		p = barline.getMatrixValue(i)
		matrix[p.y,p.x] += p.value.value()

def combine_components_into_matrix(barlines : list[Barline] | Barline, shape : tuple, imgType = np.uint8):

	if isinstance(barlines, Barline):
		barlines = [barlines]

	binmap = np.zeros(shape, imgType)

	# isRgb = len(shape) == 3
	for bl in barlines:
		append_line_to_matrix(bl, binmap)

	return binmap


class Barcode:
	def __init__(self, img : np.ndarray, build_options : barstruct = barstruct()):
		self.shape = img.shape
		self.imgType = img.dtype
		self.item = create(img, build_options)
		self.revert = build_options.proctype == ProcType.f255t0
		pass

	def get_largest_component(self):
		biggestMatrixId = 0
		lines = self.item.getBarcodeLines()
		msize = lines[0].getMatrixSize()
		for i in range(1, len(lines)):
			if lines[i].getMatrixSize() > msize:
				biggestMatrixId = i
				msize = lines[i].getMatrixSize()

		return lines[biggestMatrixId]

	def get_first_component(self):
		bar = self.item.getBarcodeLines()
		return bar[0]

	def get_first_component_points(self):
		bar = self.item.getBarcodeLines()
		if len(bar) > 0:
			return np.array(self.item.getBarcodeLines()[0].getPoints())
		else:
			return np.array([])

	def restore(self):
		binmap = combine_components_into_matrix(self.item.getBarcodeLines(), self.shape, self.imgType)

		if not self.revert:
			binmap = 255 - binmap

		return binmap

	def getComponents(self) -> set[Barline]:
		return self.item.getBarcodeLines()

	def split_components(self, threshold : int) -> tuple[np.array, np.array]:
		if threshold > len(self.item.getBarcodeLines()):
			a = combine_components_into_matrix(self, self.item.getBarcodeLines(), self.shape, self.imgType)
			b = np.zeros(self.shape, self.imgType)
			return (a, b)

		a = self.item.getBarcodeLines()[:threshold]
		b = self.item.getBarcodeLines()[threshold:]
		a = combine_components_into_matrix(self, a, self.shape, self.imgType)
		b = combine_components_into_matrix(self, b, self.shape, self.imgType)
		return (a, b)

	def combine_components_into_matrix(self):
		return combine_components_into_matrix(self.item.getBarcodeLines(), self.shape, self.imgType)

	def filter(self, LL = 180):
		bar = self.item.getBarcodeLines()

		binmap = np.zeros(self.shape, np.uint8)
		for bl in bar:

			if bl.len() < LL:
				continue

			append_line_to_matrix(bl, binmap)

		if not self.revert:
			binmap = 255 - binmap

		return binmap

	def segmentation(self, useBinarySegment = True):
		bar = self.item.getBarcodeLines()

		# red=(0,0,255)
		# blue =(255,0,0)
		# green=(0,255,0)
		# colors=[red, blue, green]

		from random import randint
		colors = []
		if not useBinarySegment:
			for i in range(len(bar)):
				colors.append(np.array([randint(0, 255),randint(0, 255),randint(0, 255)]))

			binmap = np.zeros((self.shape[0],self.shape[1],3), np.uint8)
		else:
			binmap = np.zeros((self.shape[0],self.shape[1]), np.uint8)

		i=0
		for bl in bar:
			keyvals = bl.getPoints()

			if bl.len() < 40: #and len(keyvals)<500:
				continue

			if (len(keyvals)>self.shape[0]*self.shape[1]*0.9):
				continue

			for p in keyvals:
				binmap[p.y,p.x] = 255 if useBinarySegment else colors[i%len(colors)]

			i+=1

		return binmap



def create_barcode(img, struct : barstruct):
	return Barcode(img, struct)
